# Generated content DO NOT EDIT
def eisner(scores, stn_length, remove_root=False):
    """
    Decode with Eisner's algorithm
    """
    pass

def get_entities(tags):
    """
    Convert Tags to Entities
    """
    pass

def viterbi_decode_postprocess(history, last_tags, stn_length, labels_num):
    """
    Viterbi Decode Postprocessing
    """
    pass

class Hook:
    def __init__(self):
        pass
    def add_word(self, word, freq=None):
        """
        add words to the hook, the freq can be zero
        """
        pass
    def hook(self, sentence, words):
        """
        hook to the new words
        """
        pass

class StnSplit:
    def __init__(self):
        pass
    def batch_split(self, batch_text, threads=8):
        """
        batch split to sentences
        """
        pass
    @property
    def bracket_as_entity(self):
        """
        Get the value of the bracket_as_entity option.
        """
        pass
    @property
    def en_quote_as_entity(self):
        """
        Get the value of the en_quote_as_entity option.
        """
        pass
    def split(self, text):
        """
        split to sentences
        """
        pass
    @property
    def use_en(self):
        """
        Get the value of the use_en option.
        """
        pass
    @property
    def use_zh(self):
        """
        Get the value of the use_zh option.
        """
        pass
    @property
    def zh_quote_as_entity(self):
        """
        Get the value of the zh_quote_as_entity option.
        """
        pass



import functools
import os
import time


def clock(func):
    """this is outer clock function."""

    @functools.wraps(func)  # --> 4
    def clocked(*args, **kwargs):  # -- 1
        """this is inner clocked function."""
        start_time = time.time()
        result = func(*args, **kwargs)  # --> 2
        time_cost = time.time() - start_time
        print(func.__name__ + f"({time_cost}s)")
        return result

    return clocked  # --> 3


def convert2raw(input_file, output_file):
    with open(input_file) as fi:
        with open(output_file, "w") as fo:
            for line in fi:
                line = line.strip()
                line = line.split()
                line = "".join(line)
                fo.write(line + "\n")


# ======================= 结巴分词 =======================


@clock
def jieba_load():
    import jieba

    jieba.initialize()
    return jieba


@clock
def jieba_cut(model, sentences):
    return [list(model.cut(sentence)) for sentence in sentences]


# ======================= PKUSEG =======================


@clock
def pkuseg_load():
    import pkuseg

    return pkuseg.pkuseg()


@clock
def pkuseg_cut(model, sentences):
    return [model.cut(sentence) for sentence in sentences]


# ======================= THULAC =======================
@clock
def thulac_load():
    import thulac

    thulac = thulac.thulac(seg_only=True)
    return thulac


@clock
def thulac_cut(model, sentences):
    return [model.cut(sentence, text=True) for sentence in sentences]


@clock
def thulac_fast_cut(model, sentences):
    return [model.fast_cut(sentence, text=True) for sentence in sentences]


# ======================= LTP 3 =======================


@clock
def ltp3_cws_load(model_path="../../data/legacy-models-v3/cws.model"):
    from pyltp import Segmentor

    model = Segmentor(model_path)
    return model


@clock
def ltp3_cut(model, sentences):
    return [list(model.segment(sentence)) for sentence in sentences]


# ======================= LTP =======================


@clock
def ltp_cws_load(model_path="../../data/legacy-models/cws_model.bin"):
    from ltp_extension.perceptron import CWSModel

    model = CWSModel.load(model_path)
    return model


@clock
def ltp_cut(model, sentences, threads=1):
    return model.batch_predict(sentences, threads)


# ==================== Benchmark ====================


def cws_benchmark(input_file, output_dir):
    with open(input_file) as fi:
        sentences = [line.strip() for line in fi.readlines()]
        sentences = [sentence for sentence in sentences if sentence]

    jieba_model = jieba_load()
    res = jieba_cut(jieba_model, sentences)
    with open(os.path.join(output_dir, "jieba.txt"), "w") as fo:
        for sentence in res:
            fo.write(" ".join(sentence) + "\n")

    pkuseg = pkuseg_load()
    res = pkuseg_cut(pkuseg, sentences)
    with open(os.path.join(output_dir, "pkuseg.txt"), "w") as fo:
        for sentence in res:
            fo.write(" ".join(sentence) + "\n")

    thulac_model = thulac_load()
    res = thulac_fast_cut(thulac_model, sentences)
    with open(os.path.join(output_dir, "thulac.txt"), "w") as fo:
        for sentence in res:
            fo.write(sentence + "\n")

    ltp3_model = ltp3_cws_load()
    res = ltp3_cut(ltp3_model, sentences)
    with open(os.path.join(output_dir, "ltp3.txt"), "w") as fo:
        for sentence in res:
            fo.write(" ".join(sentence) + "\n")

    ltp_model = ltp_cws_load()
    res = ltp_cut(ltp_model, sentences, threads=16)
    with open(os.path.join(output_dir, "ltp.txt"), "w") as fo:
        for sentence in res:
            fo.write(" ".join(sentence) + "\n")


def main():
    output_dir = "../../data/icwb2-data/predict/pku"
    os.makedirs(output_dir, exist_ok=True)
    cws_benchmark("../../data/icwb2-data/testing/pku_test.utf8", output_dir)

    output_dir = "../../data/icwb2-data/predict/msr"
    os.makedirs(output_dir, exist_ok=True)
    cws_benchmark("../../data/icwb2-data/testing/msr_test.utf8", output_dir)


if __name__ == "__main__":
    main()



import functools
import time


def clock(func):
    """this is outer clock function."""

    @functools.wraps(func)  # --> 4
    def clocked(*args, **kwargs):  # -- 1
        """this is inner clocked function."""
        start_time = time.time()
        result = func(*args, **kwargs)  # --> 2
        time_cost = time.time() - start_time
        print(func.__name__ + f"({time_cost}s)")
        return result

    return clocked  # --> 3


def convert2raw(input_file, output_file):
    with open(input_file) as fi:
        with open(output_file, "w") as fo:
            for line in fi:
                line = line.strip()
                line = line.split()
                line = "".join(line)
                fo.write(line + "\n")


# ======================= 结巴分词 =======================


@clock
def jieba_load():
    import jieba

    jieba.initialize()
    return jieba


@clock
def jieba_cut(model, sentences):
    return [list(model.cut(sentence)) for sentence in sentences]


# ======================= PKUSEG =======================


@clock
def pkuseg_load():
    import pkuseg

    return pkuseg.pkuseg()


@clock
def pkuseg_cut(model, sentences):
    return [model.cut(sentence) for sentence in sentences]


# ======================= THULAC =======================
@clock
def thulac_load():
    import thulac

    thulac = thulac.thulac(seg_only=True)
    return thulac


@clock
def thulac_cut(model, sentences):
    return [model.cut(sentence, text=True) for sentence in sentences]


@clock
def thulac_fast_cut(model, sentences):
    return [model.fast_cut(sentence, text=True) for sentence in sentences]


# ======================= LTP 3 =======================


@clock
def ltp3_cws_load(model_path="../../data/legacy-models-v3/cws.model"):
    from pyltp import Segmentor

    model = Segmentor(model_path)
    return model


@clock
def ltp3_cut(model, sentences):
    return [list(model.segment(sentence)) for sentence in sentences]


@clock
def ltp3_load(
    cws_model_path="../../data/legacy-models-v3/cws.model",
    pos_model_path="../../data/legacy-models-v3/pos.model",
    ner_model_path="../../data/legacy-models-v3/ner.model",
):
    from pyltp import NamedEntityRecognizer, Postagger, Segmentor

    cws_model = Segmentor(cws_model_path)
    pos_model = Postagger(pos_model_path)
    ner_model = NamedEntityRecognizer(ner_model_path)
    return cws_model, pos_model, ner_model


@clock
def ltp3_pipeline(cws_model, pos_model, ner_model, sentences):
    batch_words = [cws_model.segment(sentence) for sentence in sentences]
    batch_pos = [pos_model.postag(words) for words in batch_words]
    batch_ner = [ner_model.recognize(words, pos) for (words, pos) in zip(batch_words, batch_pos)]

    return batch_words, batch_pos, batch_ner


# ======================= LTP =======================


@clock
def ltp_cws_load(model_path="../../data/legacy-models/cws_model.bin"):
    from ltp_extension.perceptron import CWSModel

    model = CWSModel.load(model_path)
    return model


@clock
def ltp_cut(model, sentences, threads=1):
    return model.batch_predict(sentences, threads)


@clock
def ltp_load(
    cws_model_path="../../data/legacy-models/cws_model.bin",
    pos_model_path="../../data/legacy-models/pos_model.bin",
    ner_model_path="../../data/legacy-models/ner_model.bin",
):
    from ltp_extension.perceptron import CWSModel, NERModel, POSModel

    cws_model = CWSModel.load(cws_model_path)
    pos_model = POSModel.load(pos_model_path)
    ner_model = NERModel.load(ner_model_path)
    return cws_model, pos_model, ner_model


@clock
def ltp_pipeline(cws_model, pos_model, ner_model, sentences, threads=1):
    batch_words = cws_model.batch_predict(sentences, threads)
    batch_pos = pos_model.batch_predict(batch_words, threads)
    batch_ner = ner_model.batch_predict(batch_words, batch_pos, threads)

    return batch_words, batch_pos, batch_ner


# ==================== Benchmark ====================


def cws_benchmark():
    with open("../../data/benchmark/pku-all.txt") as fi:
        sentences = [line.strip() for line in fi.readlines()]

    # 0.7831432819366455s
    jieba_model = jieba_load()
    # 37.776106119155884s
    jieba_cut(jieba_model, sentences)

    # 9.232479095458984s
    pkuseg = pkuseg_load()
    # 315.90687012672424s
    pkuseg_cut(pkuseg, sentences)

    # 0.9187619686126709s
    thulac_model = thulac_load()
    # 720.1864020824432s
    thulac_cut(thulac_model, sentences)
    # 30.58756685256958s
    thulac_fast_cut(thulac_model, sentences)

    # 0.17265582084655762s
    ltp3_model = ltp3_cws_load()
    # 76.8230140209198s
    ltp3_cut(ltp3_model, sentences)

    # 1.3583240509033203s
    ltp_model = ltp_cws_load()
    # 21.614962100982666s
    ltp_cut(ltp_model, sentences, threads=1)
    # 12.079923868179321s
    ltp_cut(ltp_model, sentences, threads=2)
    # 7.003376007080078s
    ltp_cut(ltp_model, sentences, threads=4)
    # 5.0945048332214355s
    ltp_cut(ltp_model, sentences, threads=8)
    # 4.475361108779907s
    ltp_cut(ltp_model, sentences, threads=16)


def pipeline_benchmark():
    with open("../../data/benchmark/pku-all.txt") as fi:
        sentences = [line.strip() for line in fi.readlines()]

    # 11.260580062866211s
    cws_model, pos_model, ner_model = ltp_load()

    # multi threads
    # 68.13384127616882s
    ltp_pipeline(cws_model, pos_model, ner_model, sentences, threads=1)
    # 38.54593205451965s
    ltp_pipeline(cws_model, pos_model, ner_model, sentences, threads=2)
    # 21.6906418800354s
    ltp_pipeline(cws_model, pos_model, ner_model, sentences, threads=4)
    # 15.286723136901855s
    ltp_pipeline(cws_model, pos_model, ner_model, sentences, threads=8)
    # 14.134387016296387s
    ltp_pipeline(cws_model, pos_model, ner_model, sentences, threads=16)

    # 0.8289380073547363s
    cws_model_3, pos_model_3, ner_model_3 = ltp3_load()
    # 226.40391397476196s
    ltp3_pipeline(cws_model_3, pos_model_3, ner_model_3, sentences)


def main():
    # cws_benchmark()
    pipeline_benchmark()


if __name__ == "__main__":
    main()



# Generated content DO NOT EDIT
from .. import perceptron

Algorithm = perceptron.Algorithm
CWSModel = perceptron.CWSModel
CWSTrainer = perceptron.CWSTrainer
CharacterType = perceptron.CharacterType
Model = perceptron.Model
ModelType = perceptron.ModelType
NERModel = perceptron.NERModel
NERTrainer = perceptron.NERTrainer
POSModel = perceptron.POSModel
POSTrainer = perceptron.POSTrainer
Trainer = perceptron.Trainer



from ltp_extension.perceptron import Algorithm, CWSModel, CWSTrainer, Model, ModelType, Trainer


def train_cws():
    ap = Algorithm("AP")
    pa = Algorithm("Pa")
    pai = Algorithm("PaI", 0.5)
    paii = Algorithm("PaII", 0.5)

    trainer: CWSTrainer = CWSTrainer()
    trainer.epoch = 10
    trainer.load_train_data("data/cws/val.txt")
    trainer.load_eval_data("data/cws/test.txt")
    print(trainer)

    for algorithm in [ap, pa, pai, paii]:
        print(algorithm)
        trainer.algorithm = algorithm
        _: CWSModel = trainer.train()


def train_auto():
    ap = Algorithm("AP")
    pa = Algorithm("Pa")
    pai = Algorithm("PaI", 0.5)
    paii = Algorithm("PaII", 0.5)

    model_type = ModelType("cws")
    trainer: Trainer = Trainer(model_type)
    trainer.epoch = 10
    trainer.load_train_data("data/cws/val.txt")
    trainer.load_eval_data("data/cws/test.txt")
    print(trainer)

    for algorithm in [ap, pa, pai, paii]:
        print(algorithm)
        trainer.algorithm = algorithm
        _: CWSModel = trainer.train()


def main():
    # train_cws()
    train_auto()


if __name__ == "__main__":
    main()



# Generated content DO NOT EDIT
class Algorithm:
    """
    The perceptron algorithm.
    algorithm support "AP", "Pa", "PaI", "PaII"
    AP: average perceptron, param is the threads
    PA: parallel average perceptron, param is c(margin)
    """

    def __init__(self, algorithm, param=None):
        pass

class CWSModel:
    def __init__(self, path):
        pass
    def add_feature_rule(self, core, feature, s, b, m, e):
        """
        自定义新feature
        """
        pass
    def batch_predict(self, batch_text, parallelism=True):
        """
        Predict batched sentences
        """
        pass
    def disable_feature_rule(self, core, feature, s, b, m, e):
        """
        移除自定义新 feature
        """
        pass
    def disable_type_rule(self, a, b):
        """
        关闭连续不同类型之间的强制连接/切分
        """
        pass
    def disable_type_rule_d(self, a, b):
        """
        关闭连续不同类型之间的强制连接/切分(双向)
        """
        pass
    def enable_feature_rule(self, core, feature):
        """
        启用自定义新 feature
        """
        pass
    def enable_type_concat(self, a, b):
        """
        开启连续不同类型之间的强制连接
        """
        pass
    def enable_type_concat_d(self, a, b):
        """
        开启连续不同类型之间的强制连接(双向)
        """
        pass
    def enable_type_cut(self, a, b):
        """
        开启连续不同类型之间的强制切分
        """
        pass
    def enable_type_cut_d(self, a, b):
        """
        开启连续不同类型之间的强制切分(双向)
        """
        pass
    @staticmethod
    def load(path):
        """
        Load Model from a path
        """
        pass
    def predict(self, text):
        """
        Predict a sentence
        """
        pass
    def save(self, path):
        """
        Save Model to a path
        """
        pass

class CWSTrainer:
    def __init__(self):
        pass
    @property
    def algorithm(self):
        """
        Get the value of the algorithm parameter.
        """
        pass
    @property
    def compress(self):
        """
        Get the value of the compress parameter.
        """
        pass
    @property
    def epoch(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    def eval(self, model):
        """
        Eval a Segmentor model
        """
        pass
    @property
    def eval_threads(self):
        """
        Get the value of the eval_threads parameter.
        """
        pass
    def load_eval_data(self, path):
        """
        Load Eval Data from a path
        """
        pass
    def load_train_data(self, path):
        """
        Load Train Data from a path
        """
        pass
    @property
    def ratio(self):
        """
        Get the value of the ratio parameter.
        """
        pass
    @property
    def shuffle(self):
        """
        Get the value of the shuffle parameter.
        """
        pass
    @property
    def threshold(self):
        """
        Get the value of the threshold parameter.
        """
        pass
    def train(self):
        """
        Train a Segmentor model
        """
        pass
    @property
    def verbose(self):
        """
        Get the value of the verbose parameter.
        """
        pass

class CharacterType:
    """
    Digit: Digit character. (e.g. 0, 1, 2, ...)
    Roman: Roman character. (e.g. A, B, C, ...)
    Hiragana: Japanese Hiragana character. (e.g. あ, い, う, ...)
    Katakana: Japanese Katakana character. (e.g. ア, イ, ウ, ...)
    Kanji: Kanji (a.k.a. Hanzi or Hanja) character. (e.g. 漢, 字, ...)
    Other: Other character.
    """

class Model:
    def __init__(self, path, model_type=ModelType.Auto):
        pass
    def batch_predict(self, *args, parallelism=True):
        """
        Predict batched sentences
        """
        pass
    @staticmethod
    def load(path, model_type=ModelType.Auto):
        """
        Load Model from a path
        """
        pass
    def predict(self, *args):
        """
        Predict a sentence
        """
        pass
    def save(self, path):
        """
        Save Model to a path
        """
        pass
    def specialize(self):
        """
        Specialize the Model
        """
        pass

class ModelType:
    def __init__(self, model_type=None):
        pass

class NERModel:
    def __init__(self, path):
        pass
    def batch_predict(self, batch_words, batch_pos, parallelism=True):
        """
        Predict batched sentences
        """
        pass
    @staticmethod
    def load(path):
        """
        Load Model from a path
        """
        pass
    def predict(self, words, pos):
        """
        Predict a sentence
        """
        pass
    def save(self, path):
        """
        Save Model to a path
        """
        pass

class NERTrainer:
    def __init__(self, labels):
        pass
    @property
    def algorithm(self):
        """
        Get the value of the algorithm parameter.
        """
        pass
    @property
    def compress(self):
        """
        Get the value of the compress parameter.
        """
        pass
    @property
    def epoch(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    def eval(self, model):
        """
        Eval a Segmentor model
        """
        pass
    @property
    def eval_threads(self):
        """
        Get the value of the eval_threads parameter.
        """
        pass
    def load_eval_data(self, path):
        """
        Load Eval Data from a path
        """
        pass
    def load_train_data(self, path):
        """
        Load Train Data from a path
        """
        pass
    @property
    def ratio(self):
        """
        Get the value of the ratio parameter.
        """
        pass
    @property
    def shuffle(self):
        """
        Get the value of the shuffle parameter.
        """
        pass
    @property
    def threshold(self):
        """
        Get the value of the threshold parameter.
        """
        pass
    def train(self):
        """
        Train a Segmentor model
        """
        pass
    @property
    def verbose(self):
        """
        Get the value of the verbose parameter.
        """
        pass

class POSModel:
    def __init__(self, path):
        pass
    def batch_predict(self, batch_words, parallelism=True):
        """
        Predict batched sentences
        """
        pass
    @staticmethod
    def load(path):
        """
        Load Model from a path
        """
        pass
    def predict(self, words):
        """
        Predict a sentence
        """
        pass
    def save(self, path):
        """
        Save Model to a path
        """
        pass

class POSTrainer:
    def __init__(self, labels):
        pass
    @property
    def algorithm(self):
        """
        Get the value of the algorithm parameter.
        """
        pass
    @property
    def compress(self):
        """
        Get the value of the compress parameter.
        """
        pass
    @property
    def epoch(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    def eval(self, model):
        """
        Eval a Segmentor model
        """
        pass
    @property
    def eval_threads(self):
        """
        Get the value of the eval_threads parameter.
        """
        pass
    def load_eval_data(self, path):
        """
        Load Eval Data from a path
        """
        pass
    def load_train_data(self, path):
        """
        Load Train Data from a path
        """
        pass
    @property
    def ratio(self):
        """
        Get the value of the ratio parameter.
        """
        pass
    @property
    def shuffle(self):
        """
        Get the value of the shuffle parameter.
        """
        pass
    @property
    def threshold(self):
        """
        Get the value of the threshold parameter.
        """
        pass
    def train(self):
        """
        Train a Segmentor model
        """
        pass
    @property
    def verbose(self):
        """
        Get the value of the verbose parameter.
        """
        pass

class Trainer:
    def __init__(self, model_type=ModelType.Auto, labels=None):
        pass
    @property
    def algorithm(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    @property
    def compress(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    @property
    def epoch(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    def eval(self, model):
        """
        Eval a Segmentor model
        """
        pass
    @property
    def eval_threads(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    def load_eval_data(self, path):
        """
        Load Eval Data from a path
        """
        pass
    def load_train_data(self, path):
        """
        Load Train Data from a path
        """
        pass
    @property
    def ratio(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    @property
    def shuffle(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    @property
    def threshold(self):
        """
        Get the value of the epoch parameter.
        """
        pass
    def train(self):
        """
        Train a model
        """
        pass
    @property
    def verbose(self):
        """
        Get the value of the epoch parameter.
        """
        pass



# Generated content DO NOT EDIT
from .. import algorithms

eisner = algorithms.eisner
get_entities = algorithms.get_entities
viterbi_decode_postprocess = algorithms.viterbi_decode_postprocess
Hook = algorithms.Hook
StnSplit = algorithms.StnSplit
